@@ -58,6 +58,14 @@ sub new_cache {
     return CHI->new( $self->new_cache_options(), @_ );
 }
 
+sub new_cleared_cache {
+    my $self = shift;
+
+    my $cache = $self->new_cache(@_);
+    $cache->clear();
+    return $cache;
+}
+
 sub new_cache_options {
     my $self = shift;
 
@@ -646,6 +654,8 @@ sub test_l1_cache : Test(228) {
     my @values = map { "value$_" } ( 0 .. 2 );
     my ( $cache, $l1_cache );
 
+    return "skipping - no support for clear" unless $self->supports_clear();
+
     my $test_l1_cache = sub {
 
         is( $l1_cache->subcache_type, "l1_cache" );
@@ -706,7 +716,7 @@ sub test_l1_cache : Test(228) {
     # Test with current cache in primary position...
     #
     $cache =
-      $self->new_cache( l1_cache => { driver => 'Memory', global => 1 } );
+      $self->new_cache( l1_cache => { driver => 'Memory', datastore => {} } );
     $l1_cache = $cache->l1_cache;
     isa_ok( $cache,    $self->testing_driver_class );
     isa_ok( $l1_cache, 'CHI::Driver::Memory' );
@@ -715,9 +725,9 @@ sub test_l1_cache : Test(228) {
     # and in l1 position
     #
     $cache = CHI->new(
-        driver   => 'Memory',
-        global   => 1,
-        l1_cache => { $self->new_cache_options() }
+        driver    => 'Memory',
+        datastore => {},
+        l1_cache  => { $self->new_cache_options() }
     );
     $l1_cache = $cache->l1_cache;
     isa_ok( $cache,    'CHI::Driver::Memory' );
@@ -730,6 +740,8 @@ sub test_mirror_cache : Test(206) {
     my ( $cache, $mirror_cache );
     my ( $key, $value, $key2, $value2 ) = $self->kvpair(2);
 
+    return "skipping - no support for clear" unless $self->supports_clear();
+
     my $test_mirror_cache = sub {
 
         is( $mirror_cache->subcache_type, "mirror_cache" );
@@ -960,12 +972,14 @@ sub _test_common_subcache_features {
 }
 
 sub test_clear : Tests {
-    my $self  = shift;
-    my $cache = $self->{cache};
-    $self->num_tests( $self->{key_count} + 2 );
+    my $self   = shift;
+    my $cache  = $self->{cache};
+    my $cache2 = $self->new_cache( namespace => 'other' );
+    $self->num_tests( $self->{key_count} + 3 );
 
     if ( $self->supports_clear() ) {
         $self->set_some_keys($cache);
+        $self->set_some_keys($cache2);
         $cache->clear();
         cmp_deeply( [ $cache->get_keys ], [], "get_keys after clear" );
         is( scalar( $cache->get_keys ), 0, "scalar(get_keys) = 0 after clear" );
@@ -973,6 +987,11 @@ sub test_clear : Tests {
             ok( !defined $cache->get($key),
                 "key '$keyname' no longer defined after clear" );
         }
+        cmp_set(
+            [ $cache2->get_keys ],
+            [ values( %{ $self->{keys} } ) ],
+            'cache2 untouched by clear'
+        );
     }
     else {
         throws_ok(
@@ -1058,6 +1077,167 @@ sub test_cache_object : Test(6) {
     );
 }
 
+sub test_size_awareness : Test(9) {
+    my $self = shift;
+    my ( $key, $value ) = $self->kvpair();
+
+    ok( !$self->{cache}->is_size_aware(), "not size aware by default" );
+
+    my $cache = $self->new_cleared_cache( is_size_aware => 1 );
+    is( $cache->get_size(), 0, "size is 0 for empty" );
+    $cache->set( $key, $value );
+    is_about( $cache->get_size, 20, "size is about 20 with one value" );
+    $cache->set( $key, scalar( $value x 5 ) );
+    is_about( $cache->get_size, 45, "size is 45 after overwrite" );
+    $cache->set( $key, scalar( $value x 5 ) );
+    is_about( $cache->get_size, 45, "size is still 45 after same overwrite" );
+    $cache->set( $key, scalar( $value x 2 ) );
+    is_about( $cache->get_size, 26, "size is 26 after overwrite" );
+    $cache->remove($key);
+    is( $cache->get_size, 0, "size is 0 again after removing key" );
+    $cache->set( $key, $value );
+    is_about( $cache->get_size, 20, "size is about 20 with one value" );
+    $cache->clear();
+    is( $cache->get_size, 0, "size is 0 again after clear" );
+}
+
+sub test_max_size : Test(22) {
+    my $self = shift;
+
+    is( $self->new_cache( max_size => '30k' )->max_size,
+        30 * 1024, 'max_size parsing' );
+
+    my $cache = $self->new_cleared_cache( max_size => 99 );
+    ok( $cache->is_size_aware, "is size aware when max_size specified" );
+    my $value_20 = 'x' x 6;
+
+    for ( my $i = 0 ; $i < 5 ; $i++ ) {
+        $cache->set( "key$i", $value_20 );
+    }
+    for ( my $i = 0 ; $i < 10 ; $i++ ) {
+        $cache->set( "key" . int( rand(10) ), $value_20 );
+        is_between( $cache->get_size, 60, 99,
+            "after iteration $i, size = " . $cache->get_size );
+        is_between( scalar( $cache->get_keys ),
+            3, 5, "after iteration $i, keys = " . scalar( $cache->get_keys ) );
+    }
+}
+
+sub test_custom_discard_policy : Test(10) {
+    my $self          = shift;
+    my $value_20      = 'x' x 6;
+    my $highest_first = sub {
+        my $c           = shift;
+        my @sorted_keys = sort( $c->get_keys );
+        return sub { pop(@sorted_keys) };
+    };
+    my $cache = $self->new_cleared_cache(
+        is_size_aware  => 1,
+        discard_policy => $highest_first
+    );
+    for ( my $j = 0 ; $j < 10 ; $j += 2 ) {
+        $cache->clear();
+        for ( my $i = 0 ; $i < 10 ; $i++ ) {
+            my $k = ( $i + $j ) % 10;
+            $cache->set( "key$k", $value_20 );
+        }
+        $cache->discard_to_size(100);
+        cmp_set(
+            [ $cache->get_keys ],
+            [ map { "key$_" } ( 0 .. 4 ) ],
+            "5 lowest"
+        );
+        $cache->discard_to_size(20);
+        cmp_set( [ $cache->get_keys ], ["key0"], "1 lowest" );
+    }
+}
+
+sub test_discard_timeout : Test(4) {
+    my $self       = shift;
+    my $bad_policy = sub {
+        return sub { '1' };
+    };
+    my $cache = $self->new_cleared_cache(
+        is_size_aware  => 1,
+        discard_policy => $bad_policy
+    );
+    ok( defined( $cache->discard_timeout ) && $cache->discard_timeout > 1,
+        "positive discard timeout" );
+    $cache->discard_timeout(2);
+    is( $cache->discard_timeout, 2, "can set timeout" );
+    my $start_time = time;
+    $cache->set( 2, 2 );
+    throws_ok { $cache->discard_to_size(0) } qr/discard timeout .* reached/;
+    ok( time > $start_time && time < $start_time + 5,
+        "timed out in 2 seconds" );
+}
+
+sub test_size_awareness_with_subcaches : Test(19) {
+    my $self = shift;
+
+    my ( $cache, $l1_cache );
+    my $set_values = sub {
+        my $value_20 = 'x' x 6;
+        for ( my $i = 0 ; $i < 20 ; $i++ ) {
+            $cache->set( "key$i", $value_20 );
+        }
+        $l1_cache = $cache->l1_cache;
+    };
+    my $is_size_aware = sub {
+        my $c     = shift;
+        my $label = $c->label;
+
+        ok( $c->is_size_aware, "$label is size aware" );
+        my $max_size = $c->max_size;
+        ok( $max_size > 0, "$label has max size" );
+        is_between( $c->get_size, $max_size - 40,
+            $max_size, "$label size = " . $c->get_size );
+        is_between(
+            scalar( $c->get_keys ),
+            ( $max_size + 1 ) / 20 - 2,
+            ( $max_size + 1 ) / 20,
+            "$label keys = " . scalar( $c->get_keys )
+        );
+    };
+    my $is_not_size_aware = sub {
+        my $c     = shift;
+        my $label = $c->label;
+
+        ok( !$c->is_size_aware, "$label is not size aware" );
+        is( $c->get_keys, 20, "$label keys = 20" );
+    };
+
+    $cache = $self->new_cleared_cache(
+        l1_cache => { driver => 'Memory', datastore => {}, max_size => 99 } );
+    $set_values->();
+    $is_not_size_aware->($cache);
+    $is_size_aware->($l1_cache);
+
+    $cache = $self->new_cleared_cache(
+        l1_cache => { driver => 'Memory', datastore => {}, max_size => 99 },
+        max_size => 199
+    );
+    $set_values->();
+    $is_size_aware->($cache);
+    $is_size_aware->($l1_cache);
+
+    $cache = $self->new_cleared_cache(
+        l1_cache => { driver => 'Memory', datastore => {} },
+        max_size => 199
+    );
+    $set_values->();
+    $is_size_aware->($cache);
+
+    # Cannot call is_not_size_aware because the get_keys check will
+    # fail. Keys will be removed from the l1_cache when they are removed
+    # from the main cache, even though l1_cache does not have a max
+    # size. Not sure if this is the correct behavior, but for now, we're not
+    # going to test it. Normally, l1 caches will be more size limited than
+    # their parent caches.
+    #
+    ok( !$l1_cache->is_size_aware, $l1_cache->label . " is not size aware" );
+}
+
 sub is_about {
     my ( $value, $expected, $msg ) = @_;
 
@@ -1120,6 +1300,16 @@ sub test_obj_ref : Tests(8) {
     $validate_obj->($obj);
 }
 
+sub test_metacache : Tests(3) {
+    my $self  = shift;
+    my $cache = $self->{cache};
+
+    ok( !defined( $cache->{metacache} ), "metacache is lazy" );
+    $cache->metacache->set( 'foo', 5 );
+    ok( defined( $cache->{metacache} ), "metacache autovivified" );
+    is( $cache->metacache->get('foo'), 5 );
+}
+
 sub test_scalar_return_values : Tests(5) {
     my $self  = shift;
     my $cache = $self->{cache};
