@@ -2,7 +2,7 @@ package CHI::t::Util;
 use strict;
 use warnings;
 use CHI::Test;
-use CHI::Util qw(unique_id);
+use CHI::Util qw(unique_id parse_memory_size);
 use CHI::Test::Util qw(random_string);
 use List::MoreUtils qw(uniq);
 use base qw(CHI::Test::Class);
@@ -34,4 +34,17 @@ sub test_non_common_constructor_params : Tests(1) {
     cmp_deeply( $non_common_params, { map { ( $_, 1 ) } qw(foo bar baz) } );
 }
 
+sub test_parse_memory_size : Tests(5) {
+    my %results = (
+        '12345' => '12345',
+        '50K'   => 50 * 1024,
+        '8Mb'   => 8 * 1024 * 1024,
+        '301k'  => 301 * 1024,
+    );
+    while ( my ( $input, $expected ) = each(%results) ) {
+        is( parse_memory_size($input), $expected );
+    }
+    throws_ok { parse_memory_size('8kk') } qr/cannot parse/;
+}
+
 1;
